@@ -183,8 +183,29 @@ class Config(dict):
 
     def _prepare_secgroups(self):
         stack = self[A.NAME]
+        # Special magic groups for lb
+        load_balancer_groups = []
+        for lb in self.get(R.LOAD_BALANCERS, []):
+            for_servers = lb[A.loadbalancer.SERVER_NAMES]
+            for_server = filter(
+                    lambda s: s[A.server.NAME] == for_servers, 
+                    self[R.SERVERS])[0]
+            secgroup_name = '%s-secgroup' % lb[A.loadbalancer.NAME]
+            sec_group = {
+                A.secgroup.NAME: secgroup_name,
+                A.secgroup.PROVIDER: for_server[A.server.PROVIDER],
+                A.secgroup.REGION: for_server[A.server.REGION],
+                'description': "Secgroup for %s load balancer" % lb[A.loadbalancer.NAME],
+                A.secgroup.RULES: [],
+                'load_balancer': lb[A.loadbalancer.NAME],
+            }
+
+            load_balancer_groups.append(sec_group)
+            self.setdefault(R.DYNAMIC_LB_SEC_GROUPS, []).append(sec_group)
+            for_server.setdefault(A.server.STACK_SECGROUPS, []).append(secgroup_name)
 
         sg_rule_sets = []
+        self.setdefault(R.SERVER_SECURITY_GROUPS, []).extend(load_balancer_groups)
         for sg in self.get(R.SERVER_SECURITY_GROUPS, []):
             provider = sg[A.secgroup.PROVIDER]
             region = sg[A.secgroup.REGION]
@@ -201,7 +222,11 @@ class Config(dict):
 
                 r[A.secgroup.TARGET] = dressy_name
                 rules.append(r)
-            sg_rule_sets.append(
+
+            # For load balancer SGs, don't add rule sets now; 
+            # it'll get done later
+            if not sg.get('load_balancer', None):
+                sg_rule_sets.append(
                     {
                         A.secgroup.NAME: dressy_name,
                         A.secgroup.PROVIDER: provider,
@@ -269,13 +294,14 @@ class Config(dict):
                 A.VERSION: self[A.VERSION],
                 }
         for server in self.get(R.SERVERS, []):
-            # default values
-            if A.server.LAUNCH_TIMEOUT not in server:
-                server[A.server.LAUNCH_TIMEOUT] = DEFAULT_LAUNCH_TIMEOUT_S
-            if A.server.POST_DELAY not in server:
-                server[A.server.POST_DELAY] = DEFAULT_LAUNCH_TIMEOUT_S
-            if A.server.AZ not in server:
-                server[A.server.AZ] = server[A.server.REGION]
+            # default cloud values
+            if A.PROVIDER in server:
+                if A.server.LAUNCH_TIMEOUT not in server:
+                    server[A.server.LAUNCH_TIMEOUT] = DEFAULT_LAUNCH_TIMEOUT_S
+                if A.server.POST_DELAY not in server:
+                    server[A.server.POST_DELAY] = DEFAULT_LAUNCH_TIMEOUT_S
+                if A.server.AZ not in server:
+                    server[A.server.AZ] = server[A.server.REGION]
 
             # distribute the config scope attributes
             svars = {
@@ -290,6 +316,19 @@ class Config(dict):
                     svars[scope] = self[scope]
             server[A.server.VARS] = svars
 
+    def _prepare_load_balancers(self):
+        """
+        Prepare load balancer variables
+        """
+        stack = {
+                A.NAME: self[A.NAME],
+                A.VERSION: self[A.VERSION],
+                }
+
+        for load_balancer in self.get(R.LOAD_BALANCERS, []):
+            svars = {A.STACK: stack}
+            load_balancer[A.loadbalancer.VARS] = svars
+
     def prepare(self):
         """
         Reorganizes the data such that the deployment logic can find it all
@@ -322,6 +361,7 @@ class Config(dict):
         self._prepare_tags()
         self._prepare_dbs()
         self._prepare_servers()
+        self._prepare_load_balancers()
 
     def validate(self):
         """
