@@ -15,8 +15,10 @@
 # You should have received a copy of the GNU General Public License
 # along with bang.  If not, see <http://www.gnu.org/licenses/>.
 import time
-from .. import attributes as A
+from .. import resources as R, attributes as A
+from ..providers import get_provider
 from ..util import log
+from .. import BangError
 from .deployer import Deployer
 
 
@@ -160,7 +162,8 @@ class SecurityGroupRulesetDeployer(RegionedDeployer):
         """
         sg = self.consul.find_secgroup(self.name)
         current = {}
-        for rule in sg.rules:
+
+        for rule in getattr(sg, 'rules', []):
             src_group = rule.get('group')
             # TODO: allow for groups with different tenant ids.
             if src_group:
@@ -189,6 +192,7 @@ class SecurityGroupRulesetDeployer(RegionedDeployer):
             else:
                 self.create_these_rules.append(exp)
 
+
         self.delete_these_rules.extend(current.itervalues())
 
         log.debug('Create these rules: %s' % self.create_these_rules)
@@ -265,3 +269,146 @@ class DatabaseDeployer(BaseDeployer):
                 self.groups,
                 self.db_attrs
                 )
+
+
+class LoadBalancerDeployer(BaseDeployer):
+    """
+    Cloud-managed load balancer deployer. Assumes a consul able
+    to create and discover LB instances, as well as match existing
+    backend 'nodes' to a list it's given. It is assumed only a single
+    'instance' per distinct load balancer needs to be created (i.e.
+    that any elasticity is handled by the cloud service)
+    E.g config:
+      load_balancers:
+      - name: test-balancer
+        balance_server_name: server_defined_in_servers_section
+        provider: hpcloud
+        backend_port: '8080'
+        protocol: tcp
+        port: '443'
+    """
+
+    def __init__(self, *args, **kwargs):
+        super(LoadBalancerDeployer, self).__init__(*args, **kwargs)
+        self.instance_name = "%s-%s" % (self.stack.name, self.name)
+        self.lb_attrs = None
+        self.delete_these_nodes = []
+        self.add_these_nodes = []
+        self.phases = [
+                (True, self.find_existing),
+                (lambda: not self.lb_attrs, self.create),
+                (True, self.add_to_inventory),
+                (True, self.configure_nodes),
+                ]
+        self.inventory_phases = [
+                self.find_existing,
+                self.add_to_inventory,
+                ]
+
+    def find_existing(self):
+        """
+        Searches for existing load balancer instance with matching name.
+        Doesn't populate 'details' including the nodes and virtual IPs
+        """
+        self.lb_attrs = self.consul.find_lb_by_name(self.instance_name)
+
+    def create(self):
+        """Creates a new load balancer"""
+        required_nodes = self._get_required_nodes()
+        self.lb_attrs = self.consul.create_lb(
+                self.instance_name,
+                protocol=self.protocol,
+                port=self.port,
+                nodes=required_nodes,
+                node_port=str(self.backend_port),
+                algorithm=getattr(self, 'algorithm', None)
+                )
+
+    def configure_nodes(self):
+        """Ensure that the LB's nodes matches the stack"""
+        # Since load balancing runs after server provisioning,
+        # the servers should already be created regardless of 
+        # whether this was a preexisting load balancer or not.
+        # We also have the existing nodes, because add_to_inventory
+        # has been called already
+        required_nodes = self._get_required_nodes()
+
+        log.debug("Matching existing lb nodes to required %s (port %s)" %
+               (", ".join(required_nodes), self.backend_port))
+
+        self.consul.match_lb_nodes(
+            self.lb_attrs[A.loadbalancer.ID],
+            self.lb_attrs[A.loadbalancer.NODES_KEY],
+            required_nodes,
+            self.backend_port)
+
+        self.lb_attrs = self.consul.lb_details(self.lb_attrs[A.loadbalancer.ID]) 
+
+    def _get_required_nodes(self):
+        required_nodes = set()
+        for host, attrs in self.stack.groups_and_vars.dicts.items():
+            if attrs.get(A.SERVER_CLASS) == self.balance_server_name:
+                required_nodes.add(host)
+        return required_nodes
+
+
+    def add_to_inventory(self):
+        """Adds lb IPs to stack inventory"""
+        if self.lb_attrs:
+            self.lb_attrs = self.consul.lb_details(self.lb_attrs[A.loadbalancer.ID])
+            host = self.lb_attrs['virtualIps'][0]['address']
+            self.stack.add_lb_secgroup(self.name, [host], self.backend_port)
+            self.stack.add_host(
+                    host,
+                    [self.name],
+                    self.lb_attrs
+                    )
+
+class LoadBalancerSecurityGroupsDeployer(SecurityGroupRulesetDeployer):
+    def __init__(self, *args, **kwargs):
+        super(LoadBalancerSecurityGroupsDeployer, self).__init__(*args, **kwargs)
+        self.group = None
+        self.attrs = {}
+
+    def find_existing(self):
+        # Prepopulate rules from the LB stack variables
+        lb_entry = self.stack.lb_sec_groups.dicts.get(self.load_balancer)
+        if not lb_entry:
+            raise Exception, \
+                "No load balancer host found in stack for '%s'" % self.load_balancer
+        for host in lb_entry['hosts']:
+            # Create a rule for this LB. Need a mask or nova interprets it 
+            # as a group rule rather than IP rule
+            host = host + "/32"
+            rule = {
+                    A.secgroup.PROTOCOL: 'tcp',
+                    A.secgroup.FROM: lb_entry['port'],
+                    A.secgroup.TO: lb_entry['port'],
+                    A.secgroup.SOURCE: host,
+                   }       
+            self.rules.append(rule)
+
+        super(LoadBalancerSecurityGroupsDeployer, self).find_existing()
+
+DEPLOYER_MAP = {
+        R.SSH_KEYS: SSHKeyDeployer,
+        R.SERVERS: ServerDeployer,
+        R.SERVER_SECURITY_GROUPS: SecurityGroupDeployer,
+        R.SERVER_SECURITY_GROUP_RULES: SecurityGroupRulesetDeployer,
+        R.BUCKETS: BucketDeployer,
+        R.DATABASES: DatabaseDeployer,
+        R.LOAD_BALANCERS: LoadBalancerDeployer,
+        R.DYNAMIC_LB_SEC_GROUPS: LoadBalancerSecurityGroupsDeployer,
+        }
+
+
+def get_deployers(res_config, res_type, stack, creds):
+    pname = res_config[A.PROVIDER]
+    provider = get_provider(pname, creds[pname])
+    consul = provider.get_consul(res_type)
+    if not consul:
+        log.warn("%s does not provide %s" % (pname, res_type))
+        return
+    deployer = DEPLOYER_MAP[res_type]
+    count = res_config.get('instance_count', 1)
+    return [deployer(stack, res_config, consul) for _ in range(count)]
